searchData={"items":[{"type":"module","title":"Pageantry.Cast","doc":"Helper functions for casting user filter input to schema-defined types\n(which can then be used in ecto filter queries).","ref":"Pageantry.Cast.html"},{"type":"function","title":"Pageantry.Cast.cast_to_binary/3","doc":"Casts the specified value to binary.","ref":"Pageantry.Cast.html#cast_to_binary/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_binary/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_binary(true, prefs)\n    \"true\"\n    iex> cast_to_binary(-1.9, prefs)\n    \"-1.9\"\n    iex> cast_to_binary(:atom, prefs)\n    \"atom\"\n    iex> cast_to_binary(\"foo\", prefs)\n    \"foo\"\n    iex> cast_to_binary([1,2,3], prefs)\n    <<1, 2, 3>>\n    iex> cast_to_binary({1,2,3}, prefs)\n    nil","ref":"Pageantry.Cast.html#cast_to_binary/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_boolean/3","doc":"Casts the specified value to a boolean.","ref":"Pageantry.Cast.html#cast_to_boolean/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_boolean/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_boolean(true, prefs)\n    true\n    iex> cast_to_boolean(\"TRUE\", prefs)\n    true\n    iex> cast_to_boolean(\"f\", prefs)\n    false\n    iex> cast_to_boolean(\"yes\", prefs)\n    true\n    iex> cast_to_boolean(\"?\", prefs)\n    nil\n    iex> cast_to_boolean(1.5, prefs)\n    true\n    iex> cast_to_boolean(nil, true, prefs)\n    true","ref":"Pageantry.Cast.html#cast_to_boolean/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_date/3","doc":"Casts the specified value to a date.","ref":"Pageantry.Cast.html#cast_to_date/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_date/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{now: fn _ -> {:ok, ~U[2001-02-03 04:05:06Z]} end}\n    iex> cast_to_date(~D[2001-02-03], prefs)\n    ~D[2001-02-03]\n    iex> cast_to_date(~N[2001-02-03 04:05:06], prefs)\n    ~D[2001-02-03]\n    iex> cast_to_date(~U[2001-02-03 04:05:06Z], prefs)\n    ~D[2001-02-03]\n    iex> cast_to_date(%{first: ~D[2001-02-03], last: ~D[2001-02-04]}, prefs)\n    Date.range(~D[2001-02-03], ~D[2001-02-04])\n    iex> cast_to_date(\"today\", prefs)\n    Date.range(~D[2001-02-03], ~D[2001-02-03])","ref":"Pageantry.Cast.html#cast_to_date/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_datetime/3","doc":"Casts the specified value to a datetime.","ref":"Pageantry.Cast.html#cast_to_datetime/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_datetime/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{tz: \"Etc/UTC\", now: fn _ -> {:ok, ~U[2001-02-03 04:05:06Z]} end}\n    iex> cast_to_datetime(~N[2001-02-03 04:05:06], prefs)\n    ~U[2001-02-03 04:05:06Z]\n    iex> cast_to_datetime(~U[2001-02-03 04:05:06Z], prefs)\n    ~U[2001-02-03 04:05:06Z]\n    iex> cast_to_datetime(~D[2001-02-03], prefs)\n    ~U[2001-02-03 00:00:00Z]\n    iex> cast_to_datetime(%{first: ~D[2001-02-03], last: ~D[2001-02-04]}, prefs)\n    %{first: ~U[2001-02-03 00:00:00Z], last: ~U[2001-02-04 00:00:00Z]}\n    iex> cast_to_datetime(\"today\", prefs)\n    %{first: ~U[2001-02-03 00:00:00Z], last: ~U[2001-02-03 23:59:59Z]}","ref":"Pageantry.Cast.html#cast_to_datetime/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_decimal/3","doc":"Casts the specified value to a decimal.","ref":"Pageantry.Cast.html#cast_to_decimal/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_decimal/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_decimal(1, prefs)\n    Decimal.new(1)\n    iex> cast_to_decimal(-1.9, prefs)\n    Decimal.new(\"-1.9\")\n    iex> cast_to_decimal(%{first: 1.0, last: 9.0}, prefs)\n    %{first: Decimal.from_float(1.0), last: Decimal.from_float(9.0)}\n    iex> cast_to_decimal(\"-1.9\", prefs)\n    Decimal.new(\"-1.9\")\n    iex> cast_to_decimal(\"1e1\", prefs)\n    Decimal.new(\"1E+1\")\n    iex> cast_to_decimal(\"1-9\", prefs)\n    %{first: Decimal.new(1), last: Decimal.new(9)}\n    iex> cast_to_decimal(\"-9 to -1.9\", prefs)\n    %{first: Decimal.new(-9), last: Decimal.from_float(-1.9)}\n    iex> cast_to_decimal(\"?\", prefs)\n    nil\n    iex> cast_to_decimal(nil, Decimal.new(3), prefs)\n    Decimal.new(3)","ref":"Pageantry.Cast.html#cast_to_decimal/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_float/3","doc":"Casts the specified value to a float.","ref":"Pageantry.Cast.html#cast_to_float/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_float/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_float(1, prefs)\n    1.0\n    iex> cast_to_float(-1.9, prefs)\n    -1.9\n    iex> cast_to_float(%{first: 1.0, last: 9.0}, prefs)\n    %{first: 1.0, last: 9.0}\n    iex> cast_to_float(\"-1.9\", prefs)\n    -1.9\n    iex> cast_to_float(\"1e1\", prefs)\n    10.0\n    iex> cast_to_float(\"1-9\", prefs)\n    %{first: 1.0, last: 9.0}\n    iex> cast_to_float(\"-9 to -1.9\", prefs)\n    %{first: -9.0, last: -1.9}\n    iex> cast_to_float(\"?\", prefs)\n    nil\n    iex> cast_to_float(nil, 3.0, prefs)\n    3.0","ref":"Pageantry.Cast.html#cast_to_float/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_inet/3","doc":"Casts the specified value to a `Postgrex.INET`.","ref":"Pageantry.Cast.html#cast_to_inet/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_inet/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_inet(\"0.0.0.0\", prefs)\n    %Postgrex.INET{address: {0,0,0,0}}\n    iex> cast_to_inet(\"::\", prefs)\n    %Postgrex.INET{address: {0,0,0,0,0,0,0,0}}\n    iex> cast_to_inet(\"0.0.0.0/0\", prefs)\n    %Postgrex.INET{address: {0,0,0,0}, netmask: 0}\n    iex> cast_to_inet(\"1\", prefs)\n    nil","ref":"Pageantry.Cast.html#cast_to_inet/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_integer/3","doc":"Casts the specified value to an integer.","ref":"Pageantry.Cast.html#cast_to_integer/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_integer/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_integer(1, prefs)\n    1\n    iex> cast_to_integer(-1.9, prefs)\n    -1\n    iex> cast_to_integer(1..9, prefs)\n    1..9\n    iex> cast_to_integer(\"-1.9\", prefs)\n    -1\n    iex> cast_to_integer(\"1e1\", prefs)\n    10\n    iex> cast_to_integer(\"1-9\", prefs)\n    1..9\n    iex> cast_to_integer(\"-9 to -1.9\", prefs)\n    -9..-1\n    iex> cast_to_integer(\"?\", prefs)\n    nil\n    iex> cast_to_integer(nil, 3, prefs)\n    3","ref":"Pageantry.Cast.html#cast_to_integer/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_naive_datetime/3","doc":"Casts the specified value to a naive datetime.","ref":"Pageantry.Cast.html#cast_to_naive_datetime/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_naive_datetime/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{now: fn _ -> {:ok, ~U[2001-02-03 04:05:06Z]} end}\n    iex> cast_to_naive_datetime(~N[2001-02-03 04:05:06], prefs)\n    ~N[2001-02-03 04:05:06]\n    iex> cast_to_naive_datetime(~U[2001-02-03 04:05:06Z], prefs)\n    ~N[2001-02-03 04:05:06]\n    iex> cast_to_naive_datetime(~D[2001-02-03], prefs)\n    ~N[2001-02-03 00:00:00]\n    iex> cast_to_naive_datetime(%{first: ~D[2001-02-03], last: ~D[2001-02-04]}, prefs)\n    %{first: ~N[2001-02-03 00:00:00], last: ~N[2001-02-04 00:00:00]}\n    iex> cast_to_naive_datetime(\"today\", prefs)\n    %{first: ~N[2001-02-03 00:00:00], last: ~N[2001-02-03 23:59:59]}","ref":"Pageantry.Cast.html#cast_to_naive_datetime/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_string/3","doc":"Casts the specified value to a string.","ref":"Pageantry.Cast.html#cast_to_string/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_string/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{}\n    iex> cast_to_string(true, prefs)\n    \"true\"\n    iex> cast_to_string(-1.9, prefs)\n    \"-1.9\"\n    iex> cast_to_string(:atom, prefs)\n    \"atom\"\n    iex> cast_to_string(\"foo\", prefs)\n    \"foo\"\n    iex> cast_to_string([1,2,3], prefs)\n    <<1, 2, 3>>\n    iex> cast_to_string({1,2,3}, prefs)\n    nil","ref":"Pageantry.Cast.html#cast_to_string/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_time/3","doc":"Casts the specified value to a time.","ref":"Pageantry.Cast.html#cast_to_time/3"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_time/3","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{now: fn _ -> {:ok, ~U[2001-02-03 04:05:06Z]} end}\n    iex> cast_to_time(~T[04:05:06], prefs)\n    ~T[04:05:06]\n    iex> cast_to_time(~D[2001-02-03], prefs)\n    ~T[00:00:00]\n    iex> cast_to_time(~N[2001-02-03 04:05:06], prefs)\n    ~T[04:05:06]\n    iex> cast_to_time(~U[2001-02-03 04:05:06Z], prefs)\n    ~T[04:05:06]\n    iex> cast_to_time(%{first: ~T[00:00:00], last: ~T[23:59:59]}, prefs)\n    %{first: ~T[00:00:00], last: ~T[23:59:59]}\n    iex> cast_to_time(\"today\", prefs)\n    %{first: ~T[00:00:00], last: ~T[23:59:59]}","ref":"Pageantry.Cast.html#cast_to_time/3-examples"},{"type":"function","title":"Pageantry.Cast.cast_to_type/4","doc":"Casts the specified value to the specified type.","ref":"Pageantry.Cast.html#cast_to_type/4"},{"type":"function","title":"Examples - Pageantry.Cast.cast_to_type/4","doc":"iex> import Pageantry.Cast\n    iex> prefs = %Pageantry.Prefs{now: fn _ -> {:ok, ~U[2001-02-03 04:05:06Z]} end}\n    iex> cast_to_type(:boolean, \"yes\", nil, prefs)\n    true\n    iex> cast_to_type(:integer, 2.5, nil, prefs)\n    2\n    iex> cast_to_type(:integer, \"1-9\", nil, prefs)\n    1..9\n    iex> cast_to_type(:integer, \"foo\", -1, prefs)\n    -1\n    iex> cast_to_type(:utc_datetime, \"today\", nil, prefs)\n    %{first: ~U[2001-02-03 00:00:00Z], last: ~U[2001-02-03 23:59:59Z]}","ref":"Pageantry.Cast.html#cast_to_type/4-examples"},{"type":"module","title":"Pageantry.Field","doc":"Field definition for sorting/filtering.","ref":"Pageantry.Field.html"},{"type":"module","title":"Fields - Pageantry.Field","doc":"* `field`\n: Field name (in schema), eg `:inserted_at`.\n* `relation`\n: If field is not directly in schema, how the field is related.\n* `repo`\n: HACK this can be removed when `Pageantry.Page.build_subquery_condition/3`\n  no longer needs it.\n* `sort`\n: True if sortable (defaults to true).\n* `filter`\n: Filter type:\n  * `:false`: Cannot be used as filter.\n  * `:equal`: Filter value must be exactly equal to field value.\n  * `:like`: Filter value matched to field value with SQL `LIKE`.\n  * `:boolean`: Filter and field values casted to boolean and matched.\n    For cases where you want to filter on whether an optional field has been set\n    (eg \"has_address=yes\").\n* `all`\n: True if included in special `:ALL` filter (defaults to true).","ref":"Pageantry.Field.html#module-fields"},{"type":"type","title":"Pageantry.Field.filter/0","doc":"","ref":"Pageantry.Field.html#t:filter/0"},{"type":"type","title":"Pageantry.Field.t/0","doc":"","ref":"Pageantry.Field.html#t:t/0"},{"type":"module","title":"Pageantry.FieldRelation","doc":"Relation definition for sorting/filtering.","ref":"Pageantry.FieldRelation.html"},{"type":"module","title":"Fields - Pageantry.FieldRelation","doc":"* `association`\n: Association name in this schema (eg `:org`).\n* `relation`\n: Nested relation, if this relation is an intermediate step to another\n  related schema.\n* `schema`\n: Associated schema, if overriding specified association.","ref":"Pageantry.FieldRelation.html#module-fields"},{"type":"type","title":"Pageantry.FieldRelation.t/0","doc":"","ref":"Pageantry.FieldRelation.html#t:t/0"},{"type":"module","title":"Pageantry.InetType","doc":"`Ecto.Type` implementation for postgres `INET` type.","ref":"Pageantry.InetType.html"},{"type":"function","title":"Pageantry.InetType.cast/1","doc":"","ref":"Pageantry.InetType.html#cast/1"},{"type":"function","title":"Pageantry.InetType.dump/1","doc":"","ref":"Pageantry.InetType.html#dump/1"},{"type":"function","title":"Pageantry.InetType.embed_as/1","doc":"","ref":"Pageantry.InetType.html#embed_as/1"},{"type":"function","title":"Pageantry.InetType.equal?/2","doc":"","ref":"Pageantry.InetType.html#equal?/2"},{"type":"function","title":"Pageantry.InetType.load/1","doc":"","ref":"Pageantry.InetType.html#load/1"},{"type":"function","title":"Pageantry.InetType.normalize_cidr/1","doc":"Adds default netmask if nil.","ref":"Pageantry.InetType.html#normalize_cidr/1"},{"type":"function","title":"Pageantry.InetType.type/0","doc":"","ref":"Pageantry.InetType.html#type/0"},{"type":"module","title":"Pageantry.Input","doc":"User input for paging/sorting/filtering.","ref":"Pageantry.Input.html"},{"type":"module","title":"Fields - Pageantry.Input","doc":"* `off`\n: Page offset start; `0` to start with the first item.\n* `max`\n: Maximum items per page, eg `10`.\n* `sort`\n: Keyword list of fields to sort by, eg `[asc: :name, desc: :created]`.\n* `filter`\n: Keyword list of fields to filter by, eg `[name: \"foo\", active: true]`.\n  `ALL` is a special field that means filter across all fields, eg `[ALL: \"today\"]`.","ref":"Pageantry.Input.html#module-fields"},{"type":"function","title":"Pageantry.Input.add_filter_param/2","doc":"Adds filter parameter to map of paging params.","ref":"Pageantry.Input.html#add_filter_param/2"},{"type":"function","title":"Examples - Pageantry.Input.add_filter_param/2","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> add_filter_param(%{}, %Input{filter: [created: \"today\"]})\n    %{\"field\" => \"created\", \"q\" => \"today\"}\n    iex> add_filter_param(%{}, %Input{filter: [ALL: \"today\"]})\n    %{\"q\" => \"today\"}\n    iex> add_filter_param(%{}, %Input{})\n    %{}","ref":"Pageantry.Input.html#add_filter_param/2-examples"},{"type":"function","title":"Pageantry.Input.add_max_param/3","doc":"Adds items-per-page parameter to map of paging params.","ref":"Pageantry.Input.html#add_max_param/3"},{"type":"function","title":"Examples - Pageantry.Input.add_max_param/3","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> add_max_param(%{}, %Input{max: 100}, 10)\n    %{\"max\" => \"100\"}\n    iex> add_max_param(%{}, %Input{max: 100}, 100)\n    %{}","ref":"Pageantry.Input.html#add_max_param/3-examples"},{"type":"function","title":"Pageantry.Input.add_off_param/2","doc":"Adds offset parameter to map of paging params.","ref":"Pageantry.Input.html#add_off_param/2"},{"type":"function","title":"Examples - Pageantry.Input.add_off_param/2","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> add_off_param(%{}, %Input{off: 10})\n    %{\"off\" => \"10\"}\n    iex> add_off_param(%{}, %Input{off: 0})\n    %{}","ref":"Pageantry.Input.html#add_off_param/2-examples"},{"type":"function","title":"Pageantry.Input.add_params/3","doc":"Adds paging to existing params map.","ref":"Pageantry.Input.html#add_params/3"},{"type":"function","title":"Examples - Pageantry.Input.add_params/3","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> add_params(%{}, %Input{off: 10, max: 20, sort: [asc: :name], filter: [ALL: \"today\"]})\n    %{\"max\" => \"20\", \"off\" => \"10\", \"q\" => \"today\", \"sort\" => \"name\"}\n    iex> add_params(%{}, %Input{})\n    %{}","ref":"Pageantry.Input.html#add_params/3-examples"},{"type":"function","title":"Pageantry.Input.add_sort_param/2","doc":"Adds sort parameter to map of paging params.","ref":"Pageantry.Input.html#add_sort_param/2"},{"type":"function","title":"Examples - Pageantry.Input.add_sort_param/2","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> add_sort_param(%{}, %Input{sort: [asc: :name]})\n    %{\"sort\" => \"name\"}\n    iex> add_sort_param(%{}, %Input{sort: [desc: :name]})\n    %{\"sort\" => \"-name\"}\n    iex> add_sort_param(%{}, %Input{})\n    %{}","ref":"Pageantry.Input.html#add_sort_param/2-examples"},{"type":"function","title":"Pageantry.Input.new/1","doc":"Creates new Input struct.","ref":"Pageantry.Input.html#new/1"},{"type":"function","title":"Examples - Pageantry.Input.new/1","doc":"iex> import Pageantry.Input\n    iex> new(100)\n    %Pageantry.Input{off: 0, max: 100, sort: [], filter: []}","ref":"Pageantry.Input.html#new/1-examples"},{"type":"function","title":"Pageantry.Input.new/3","doc":"Creates new Input struct.","ref":"Pageantry.Input.html#new/3"},{"type":"function","title":"Examples - Pageantry.Input.new/3","doc":"iex> import Pageantry.Input\n    iex> new(100, [desc: :created], [created: \"today\"])\n    %Pageantry.Input{off: 0, max: 100, sort: [desc: :created], filter: [created: \"today\"]}","ref":"Pageantry.Input.html#new/3-examples"},{"type":"function","title":"Pageantry.Input.parse/2","doc":"Parses request parameters into Input struct.","ref":"Pageantry.Input.html#parse/2"},{"type":"function","title":"Examples - Pageantry.Input.parse/2","doc":"iex> import Pageantry.Input\n    iex> parse(%{\"off\" => \"100\"})\n    %Pageantry.Input{off: 100, max: 10, sort: [], filter: []}\n    iex> parse(%{\"sort\" => \"name-created\", \"q\" => \"today\"})\n    %Pageantry.Input{off: 0, max: 10, sort: [asc: :name, desc: :created], filter: [ALL: \"today\"]}","ref":"Pageantry.Input.html#parse/2-examples"},{"type":"function","title":"Pageantry.Input.parse/3","doc":"Parses request parameters into Input struct with defaults.","ref":"Pageantry.Input.html#parse/3"},{"type":"function","title":"Examples - Pageantry.Input.parse/3","doc":"iex> import Pageantry.Input\n    iex> parse(%Pageantry.Input{max: 20}, %{\"off\" => \"100\"}, %Pageantry.Prefs{})\n    %Pageantry.Input{off: 100, max: 20, sort: [], filter: []}\n    iex> input = %Pageantry.Input{max: 20, sort: [desc: :created]}\n    iex> params = %{\"max\" => \"100\", \"field\" => \"name\", \"q\" => \"foo\"}\n    iex> parse(input, params, %Pageantry.Prefs{})\n    %Pageantry.Input{off: 0, max: 100, sort: [desc: :created], filter: [name: \"foo\"]}","ref":"Pageantry.Input.html#parse/3-examples"},{"type":"function","title":"Pageantry.Input.parse_filter/3","doc":"Extracts \"field\" and \"q\" parameters as filter keyword list.","ref":"Pageantry.Input.html#parse_filter/3"},{"type":"function","title":"Examples - Pageantry.Input.parse_filter/3","doc":"iex> import Pageantry.Input\n    iex> parse_filter(%{\"field\" => \"created\", \"q\" => \"today\"})\n    [created: \"today\"]\n    iex> parse_filter(%{\"q\" => \"today\"})\n    [ALL: \"today\"]\n    iex> parse_filter(%{\"q\" => \"\"})\n    []","ref":"Pageantry.Input.html#parse_filter/3-examples"},{"type":"function","title":"Pageantry.Input.parse_max/3","doc":"Extracts \"max\" parameter as positive integer.","ref":"Pageantry.Input.html#parse_max/3"},{"type":"function","title":"Examples - Pageantry.Input.parse_max/3","doc":"iex> import Pageantry.Input\n    iex> parse_max(%{\"max\" => \"100\"})\n    100\n    iex> parse_max(%{\"max\" => \"\"}, 10)\n    10","ref":"Pageantry.Input.html#parse_max/3-examples"},{"type":"function","title":"Pageantry.Input.parse_off/3","doc":"Extracts \"off\" parameter as non-negative integer.","ref":"Pageantry.Input.html#parse_off/3"},{"type":"function","title":"Examples - Pageantry.Input.parse_off/3","doc":"iex> import Pageantry.Input\n    iex> parse_off(%{\"off\" => \"100\"})\n    100\n    iex> parse_off(%{\"off\" => \"\"})\n    0","ref":"Pageantry.Input.html#parse_off/3-examples"},{"type":"function","title":"Pageantry.Input.parse_sort/3","doc":"Extracts \"sort\" parameter as sort keyword list.","ref":"Pageantry.Input.html#parse_sort/3"},{"type":"function","title":"Examples - Pageantry.Input.parse_sort/3","doc":"iex> import Pageantry.Input\n    iex> parse_sort(%{\"sort\" => \"name\"})\n    [asc: :name]\n    iex> parse_sort(%{\"sort\" => \"-name\"})\n    [desc: :name]\n    iex> parse_sort(%{\"sort\" => \"\"})\n    []","ref":"Pageantry.Input.html#parse_sort/3-examples"},{"type":"function","title":"Pageantry.Input.to_params/2","doc":"Builds paging params map.","ref":"Pageantry.Input.html#to_params/2"},{"type":"function","title":"Examples - Pageantry.Input.to_params/2","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> to_params(%Input{off: 10, max: 20, sort: [asc: :name], filter: [ALL: \"today\"]})\n    %{\"max\" => \"20\", \"off\" => \"10\", \"q\" => \"today\", \"sort\" => \"name\"}\n    iex> to_params(%Input{})\n    %{}","ref":"Pageantry.Input.html#to_params/2-examples"},{"type":"function","title":"Pageantry.Input.to_url/2","doc":"Builds URL with paging params in query string.","ref":"Pageantry.Input.html#to_url/2"},{"type":"function","title":"Examples - Pageantry.Input.to_url/2","doc":"iex> import Pageantry.Input\n    iex> alias Pageantry.Input\n    iex> to_url(%Input{off: 10, max: 20, sort: [asc: :name], filter: [ALL: \"today\"]})\n    \"?max=20&off=10&q=today&sort=name\"\n    iex> to_url(%Input{})\n    \"\"","ref":"Pageantry.Input.html#to_url/2-examples"},{"type":"type","title":"Pageantry.Input.t/0","doc":"","ref":"Pageantry.Input.html#t:t/0"},{"type":"module","title":"Pageantry.NaturalDateRange","doc":"Represents a datetime (or date or time) range.","ref":"Pageantry.NaturalDateRange.html"},{"type":"module","title":"Fields - Pageantry.NaturalDateRange","doc":"* `first`\n: First datetime in range (inclusive).\n* `last`\n: Last datetime in range (inclusive).","ref":"Pageantry.NaturalDateRange.html#module-fields"},{"type":"function","title":"Pageantry.NaturalDateRange.calc_natural_date_end/3","doc":"Calculates the date end for the specified natural-date tokens and anchor.","ref":"Pageantry.NaturalDateRange.html#calc_natural_date_end/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.calc_natural_date_end/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> prefs = %Pageantry.Prefs{}\n    iex> calc_natural_date_end(%{one_word: \"today\"}, ~U[2001-02-03 04:05:06Z], prefs)\n    ~U[2001-02-03 23:59:59Z]","ref":"Pageantry.NaturalDateRange.html#calc_natural_date_end/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.calc_natural_date_start/3","doc":"Calculates the date start for the specified natural-date tokens and anchor.","ref":"Pageantry.NaturalDateRange.html#calc_natural_date_start/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.calc_natural_date_start/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> prefs = %Pageantry.Prefs{}\n    iex> calc_natural_date_start(%{one_word: \"today\"}, ~U[2001-02-03 04:05:06Z], prefs)\n    ~U[2001-02-03 00:00:00Z]","ref":"Pageantry.NaturalDateRange.html#calc_natural_date_start/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.calc_relative_date_start/3","doc":"Calculates the date start for the relative unit at the given anchor.","ref":"Pageantry.NaturalDateRange.html#calc_relative_date_start/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.calc_relative_date_start/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> calc_relative_date_start(:months, ~U[2001-02-03 04:05:06Z], %{})\n    ~U[2001-02-01 00:00:00Z]","ref":"Pageantry.NaturalDateRange.html#calc_relative_date_start/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.last_day_of_month/2","doc":"Calculates last day of the specified month.","ref":"Pageantry.NaturalDateRange.html#last_day_of_month/2"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.last_day_of_month/2","doc":"iex> import Pageantry.NaturalDateRange\n    iex> last_day_of_month(2000, 1)\n    31\n    iex> last_day_of_month(2000, 2)\n    29\n    iex> last_day_of_month(2001, 2)\n    28","ref":"Pageantry.NaturalDateRange.html#last_day_of_month/2-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.new/2","doc":"Creates a new range from the specified first and last dates.","ref":"Pageantry.NaturalDateRange.html#new/2"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.new/2","doc":"iex> import Pageantry.NaturalDateRange\n    iex> new(~D[2001-02-03], ~D[2001-02-13])\n    %{first: ~D[2001-02-03], last: ~D[2001-02-13]}","ref":"Pageantry.NaturalDateRange.html#new/2-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.normalize_relative/1","doc":"Converts the specified string relative value to an atom.","ref":"Pageantry.NaturalDateRange.html#normalize_relative/1"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.normalize_relative/1","doc":"iex> import Pageantry.NaturalDateRange\n    iex> normalize_relative(\"next\")\n    :next\n    iex> normalize_relative(\"past\")\n    :prev","ref":"Pageantry.NaturalDateRange.html#normalize_relative/1-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.normalize_token_key/1","doc":"Converts the specified string token key to an atom.","ref":"Pageantry.NaturalDateRange.html#normalize_token_key/1"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.normalize_token_key/1","doc":"iex> import Pageantry.NaturalDateRange\n    iex> normalize_token_key(\"month_name_lo\")\n    :month","ref":"Pageantry.NaturalDateRange.html#normalize_token_key/1-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.normalize_units/1","doc":"Converts the specified string units to an atom.","ref":"Pageantry.NaturalDateRange.html#normalize_units/1"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.normalize_units/1","doc":"iex> import Pageantry.NaturalDateRange\n    iex> normalize_units(\"year\")\n    :years\n    iex> normalize_units(\"mo\")\n    :months","ref":"Pageantry.NaturalDateRange.html#normalize_units/1-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.parse/3","doc":"Parses the specified natural-date string into a first date-time to last date-time range.","ref":"Pageantry.NaturalDateRange.html#parse/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.parse/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> prefs = %Pageantry.Prefs{now: fn _ -> {:ok, ~U[2001-02-03 04:05:06Z]} end}\n    iex> parse(\"foobar\", nil, prefs)\n    nil\n    iex> parse(\"today\", nil, prefs)\n    %{first: ~U[2001-02-03 00:00:00Z], last: ~U[2001-02-03 23:59:59Z]}","ref":"Pageantry.NaturalDateRange.html#parse/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.parse_hour_or_default/3","doc":"Parses hour number at map key and applies meridian, or returns default.","ref":"Pageantry.NaturalDateRange.html#parse_hour_or_default/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.parse_hour_or_default/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> parse_hour_or_default(%{meridian: \"pm\"}, :hour, 1)\n    1\n    iex> parse_hour_or_default(%{hour: \"2\"}, :hour, 1)\n    2\n    iex> parse_hour_or_default(%{hour: \"2\", meridian: \"pm\"}, :hour, 1)\n    14","ref":"Pageantry.NaturalDateRange.html#parse_hour_or_default/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.parse_map_number_or_default/3","doc":"Parses number at map key, or returns default.","ref":"Pageantry.NaturalDateRange.html#parse_map_number_or_default/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.parse_map_number_or_default/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> parse_map_number_or_default(%{}, :day, 1)\n    1\n    iex> parse_map_number_or_default(%{day: \"2\"}, :day, 1)\n    2","ref":"Pageantry.NaturalDateRange.html#parse_map_number_or_default/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.parse_month_or_default/3","doc":"Parses month number at map key, or returns default.","ref":"Pageantry.NaturalDateRange.html#parse_month_or_default/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.parse_month_or_default/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> parse_month_or_default(%{}, :month, 1)\n    1\n    iex> parse_month_or_default(%{month: \"2\"}, :month, 1)\n    2\n    iex> parse_month_or_default(%{month: \"feb\"}, :month, 1)\n    2","ref":"Pageantry.NaturalDateRange.html#parse_month_or_default/3-examples"},{"type":"function","title":"Pageantry.NaturalDateRange.parse_year_or_default/3","doc":"Parses year number at map key, or returns default.","ref":"Pageantry.NaturalDateRange.html#parse_year_or_default/3"},{"type":"function","title":"Examples - Pageantry.NaturalDateRange.parse_year_or_default/3","doc":"iex> import Pageantry.NaturalDateRange\n    iex> parse_year_or_default(%{}, :year, 1)\n    1\n    iex> parse_year_or_default(%{year: \"00\"}, :year, 1)\n    2000\n    iex> parse_year_or_default(%{year: \"70\"}, :year, 1)\n    1970","ref":"Pageantry.NaturalDateRange.html#parse_year_or_default/3-examples"},{"type":"type","title":"Pageantry.NaturalDateRange.natural_date_range_tokens/0","doc":"","ref":"Pageantry.NaturalDateRange.html#t:natural_date_range_tokens/0"},{"type":"type","title":"Pageantry.NaturalDateRange.t/0","doc":"","ref":"Pageantry.NaturalDateRange.html#t:t/0"},{"type":"module","title":"Pageantry.Output","doc":"Output of paging/sorting/filtering.","ref":"Pageantry.Output.html"},{"type":"module","title":"Fields - Pageantry.Output","doc":"* `items`\n: List of paged/sorted/filtered items.\n* `total`\n: Total filtered items available (ie combined total of items on all pages).\n  `-1` if unknown.","ref":"Pageantry.Output.html#module-fields"},{"type":"type","title":"Pageantry.Output.t/0","doc":"","ref":"Pageantry.Output.html#t:t/0"},{"type":"module","title":"Pageantry.Page","doc":"Combination of user preferences, user input, validation requirements,\nresult output, and the Ecto query for generating the output\nof a paged/sorted/filtered list.","ref":"Pageantry.Page.html"},{"type":"module","title":"Fields - Pageantry.Page","doc":"* `query`\n: Ecto query for generating output.\n* `prefs`\n: User preferences.\n* `input`\n: User input.\n* `validation`\n: Validation definitions and restrictions to apply when adding user input\n  to base query.\n* `output`\n: Output results.","ref":"Pageantry.Page.html#module-fields"},{"type":"function","title":"Pageantry.Page.count/1","doc":"Executes count query and updates output total with the result.","ref":"Pageantry.Page.html#count/1"},{"type":"function","title":"Pageantry.Page.filter_output/1","doc":"Filters the output items.","ref":"Pageantry.Page.html#filter_output/1"},{"type":"function","title":"Pageantry.Page.input/1","doc":"Creates new Page struct with input defaults.","ref":"Pageantry.Page.html#input/1"},{"type":"function","title":"Examples - Pageantry.Page.input/1","doc":"iex> import Pageantry.Page\n    iex> input(100).input\n    %Pageantry.Input{off: 0, max: 100, sort: [], filter: []}","ref":"Pageantry.Page.html#input/1-examples"},{"type":"function","title":"Pageantry.Page.input/3","doc":"Creates new Page struct with input defaults.","ref":"Pageantry.Page.html#input/3"},{"type":"function","title":"Examples - Pageantry.Page.input/3","doc":"iex> import Pageantry.Page\n    iex> input(100, name: \"foo\").input\n    %Pageantry.Input{max: 100, filter: [name: \"foo\"]}\n    iex> input(100, [desc: :created], [created: \"today\"]).input\n    %Pageantry.Input{off: 0, max: 100, sort: [desc: :created], filter: [created: \"today\"]}","ref":"Pageantry.Page.html#input/3-examples"},{"type":"function","title":"Pageantry.Page.join/1","doc":"Adds join clause from user input and validation requirements to Ecto query.","ref":"Pageantry.Page.html#join/1"},{"type":"function","title":"Pageantry.Page.limit/1","doc":"Adds limit from user input and validation requirements to Ecto query.","ref":"Pageantry.Page.html#limit/1"},{"type":"function","title":"Pageantry.Page.map/2","doc":"Maps each output item with the specified function and updates the output struct.","ref":"Pageantry.Page.html#map/2"},{"type":"function","title":"Examples - Pageantry.Page.map/2","doc":"iex> import Pageantry.Page\n    iex> page = %Pageantry.Page{output: %Pageantry.Output{items: [1, 2, 3]}}\n    iex> map(page, &to_string/1).output.items\n    [\"1\", \"2\", \"3\"]","ref":"Pageantry.Page.html#map/2-examples"},{"type":"function","title":"Pageantry.Page.new/0","doc":"Creates new Page struct with all defaults.","ref":"Pageantry.Page.html#new/0"},{"type":"function","title":"Examples - Pageantry.Page.new/0","doc":"iex> import Pageantry.Page\n    iex> new()\n    %Pageantry.Page{}","ref":"Pageantry.Page.html#new/0-examples"},{"type":"function","title":"Pageantry.Page.new/1","doc":"Creates new Page struct with specified prefs.","ref":"Pageantry.Page.html#new/1"},{"type":"function","title":"Examples - Pageantry.Page.new/1","doc":"iex> import Pageantry.Page\n    iex> new(%Pageantry.Prefs{tz: \"America/New_York\"})\n    %Pageantry.Page{prefs: %Pageantry.Prefs{tz: \"America/New_York\"}}","ref":"Pageantry.Page.html#new/1-examples"},{"type":"function","title":"Pageantry.Page.new/2","doc":"Creates new Page struct with prefs and parses params.","ref":"Pageantry.Page.html#new/2"},{"type":"function","title":"Examples - Pageantry.Page.new/2","doc":"iex> import Pageantry.Page\n    iex> page = new(%Pageantry.Prefs{tz: \"America/New_York\"}, %{\"off\" => \"100\"})\n    iex> page.prefs.tz\n    \"America/New_York\"\n    iex> page.input.off\n    100","ref":"Pageantry.Page.html#new/2-examples"},{"type":"function","title":"Pageantry.Page.offset/1","doc":"Adds offset from user input and validation requirements to Ecto query.","ref":"Pageantry.Page.html#offset/1"},{"type":"function","title":"Pageantry.Page.order_by/1","doc":"Adds order by from user input and validation requirements to Ecto query.","ref":"Pageantry.Page.html#order_by/1"},{"type":"function","title":"Pageantry.Page.parse/1","doc":"Creates new Page struct with parsed user input.","ref":"Pageantry.Page.html#parse/1"},{"type":"function","title":"Examples - Pageantry.Page.parse/1","doc":"iex> import Pageantry.Page\n    iex> parse(%{\"max\" => \"100\", \"field\" => \"name\", \"q\" => \"foo\"}).input\n    %Pageantry.Input{max: 100, filter: [name: \"foo\"]}","ref":"Pageantry.Page.html#parse/1-examples"},{"type":"function","title":"Pageantry.Page.parse/2","doc":"Adds parsed user input to page struct.","ref":"Pageantry.Page.html#parse/2"},{"type":"function","title":"Examples - Pageantry.Page.parse/2","doc":"iex> import Pageantry.Page\n    iex> parse(input(100), %{\"field\" => \"name\", \"q\" => \"foo\"}).input\n    %Pageantry.Input{max: 100, filter: [name: \"foo\"]}","ref":"Pageantry.Page.html#parse/2-examples"},{"type":"function","title":"Pageantry.Page.put_output/2","doc":"Adds output items to page struct.","ref":"Pageantry.Page.html#put_output/2"},{"type":"function","title":"Pageantry.Page.put_prefs/2","doc":"Adds user prefs to page struct.","ref":"Pageantry.Page.html#put_prefs/2"},{"type":"function","title":"Examples - Pageantry.Page.put_prefs/2","doc":"iex> import Pageantry.Page\n    iex> page = new() |> put_prefs(%Pageantry.Prefs{tz: \"America/New_York\"})\n    iex> page.prefs.weekstart\n    :sun\n    iex> page.prefs.tz\n    \"America/New_York\"","ref":"Pageantry.Page.html#put_prefs/2-examples"},{"type":"function","title":"Pageantry.Page.put_query/2","doc":"Adds Ecto query to page struct.","ref":"Pageantry.Page.html#put_query/2"},{"type":"function","title":"Pageantry.Page.put_query/3","doc":"Adds Ecto repo and query to page struct.","ref":"Pageantry.Page.html#put_query/3"},{"type":"function","title":"Pageantry.Page.put_repo/2","doc":"Adds Ecto repo to page struct.","ref":"Pageantry.Page.html#put_repo/2"},{"type":"function","title":"Pageantry.Page.put_validation/2","doc":"Adds validation definitions and restrictions to page struct.","ref":"Pageantry.Page.html#put_validation/2"},{"type":"function","title":"Examples - Pageantry.Page.put_validation/2","doc":"iex> import Pageantry.Page\n    iex> v = %Pageantry.Validation{max_max: 100, base_filter: [active: true]}\n    iex> page = new() |> put_validation(v)\n    iex> page.validation.max_off\n    0\n    iex> page.validation.max_max\n    100\n    iex> page.validation.base_filter\n    [active: true]","ref":"Pageantry.Page.html#put_validation/2-examples"},{"type":"function","title":"Pageantry.Page.query/1","doc":"Updates query in page struct with paging/sorting/filtering from input,\nexecutes query, and updates output struct with result.","ref":"Pageantry.Page.html#query/1"},{"type":"function","title":"Pageantry.Page.query/2","doc":"Updates page struct with the specified query,\nupdates query with paging/sorting/filtering from input,\nexecutes query, and updates output struct with result.","ref":"Pageantry.Page.html#query/2"},{"type":"function","title":"Pageantry.Page.query/3","doc":"Updates page struct with the specified repo and query,\nupdates query with paging/sorting/filtering from input,\nexecutes query, and updates output struct with result.","ref":"Pageantry.Page.html#query/3"},{"type":"function","title":"Pageantry.Page.query/4","doc":"Updates page struct with the specified repo and query and validation,\nupdates query with paging/sorting/filtering from input,\nexecutes query, and updates output struct with result.","ref":"Pageantry.Page.html#query/4"},{"type":"function","title":"Pageantry.Page.query/5","doc":"Updates page struct with the specified repo and query and validation\nand request parameters, updates query with paging/sorting/filtering from input,\nexecutes query, and updates output struct with result.","ref":"Pageantry.Page.html#query/5"},{"type":"function","title":"Pageantry.Page.query/6","doc":"Updates page struct with the specified repo and query and validation\nand request parameters and user prefs,\nupdates query with paging/sorting/filtering from input,\nexecutes query, and updates output struct with result.","ref":"Pageantry.Page.html#query/6"},{"type":"function","title":"Pageantry.Page.select/1","doc":"Executes select query and updates output items with the result.","ref":"Pageantry.Page.html#select/1"},{"type":"function","title":"Pageantry.Page.slice/1","doc":"Filters, sorts, and slices output items.","ref":"Pageantry.Page.html#slice/1"},{"type":"function","title":"Pageantry.Page.slice/2","doc":"Updates page struct with the specified output items, and filters, sort, and slices the items.","ref":"Pageantry.Page.html#slice/2"},{"type":"function","title":"Pageantry.Page.slice/3","doc":"Updates page struct with the specified output items and validation,\nand filters, sort, and slices the items.","ref":"Pageantry.Page.html#slice/3"},{"type":"function","title":"Pageantry.Page.slice/4","doc":"Updates page struct with the specified output items and validation and request parameters,\nand filters, sort, and slices the items.","ref":"Pageantry.Page.html#slice/4"},{"type":"function","title":"Pageantry.Page.slice_output/1","doc":"Applies offset and max length to the output items.","ref":"Pageantry.Page.html#slice_output/1"},{"type":"function","title":"Pageantry.Page.sort_output/1","doc":"Sorts the output items.","ref":"Pageantry.Page.html#sort_output/1"},{"type":"function","title":"Pageantry.Page.where/1","doc":"Adds where clause from user input and validation requirements to Ecto query.","ref":"Pageantry.Page.html#where/1"},{"type":"type","title":"Pageantry.Page.t/0","doc":"","ref":"Pageantry.Page.html#t:t/0"},{"type":"module","title":"Pageantry.Prefs","doc":"User preferences for paging/sorting/filtering.","ref":"Pageantry.Prefs.html"},{"type":"module","title":"Fields - Pageantry.Prefs","doc":"* `tz`\n: Time zone, eg \"Etc/UTC\" or \"America/Los_Angeles\".\n* `weekstart`\n: Day the week starts on, eg `:sun` or `:mon`.\n* `now`\n: Function that generates the current `DateTime` for testing.\n  Defaults to `&DateTime.now/1`.","ref":"Pageantry.Prefs.html#module-fields"},{"type":"type","title":"Pageantry.Prefs.t/0","doc":"","ref":"Pageantry.Prefs.html#t:t/0"},{"type":"module","title":"Pageantry.Validation","doc":"Definitions and restrictions for paging/sorting/filtering.","ref":"Pageantry.Validation.html"},{"type":"module","title":"Fields - Pageantry.Validation","doc":"* `schema`\n: Ecto schema to be paged.\n* `max_off`\n: Maximum page offset, `0` for unlimited (the default).\n* `max_max`\n: Maximum max-items-per-page, eg `1000`.\n* `max_pages`\n: Maximum pages to display in UI paging control, eg `10` for a control like\n  \"<< 1 2 3 4 5 6 7 8 9 10 >>\".\n* `base_sort`\n: Sort always applied after user-supplied sort, eg `[asc: :id]` to always\n  sort by `id` field as the last item of the SQL `ORDER` clause.\n* `base_filter`\n: Filter always applied in addition to user-supplied filter, eg `[active: true]`\n  to restrict results to active items only.\n* `fields`\n: Keyword list of fields that can be used for sorting/filtering.\n  The field keywords represent the name of the field exposed to the user,\n  and don't have to match the field name in the schema. For example,\n  `user_id` can be used for the `id` field of the `User` schema and\n  `org_id` can be used for the `id` field of the `Organization` schema, etc.","ref":"Pageantry.Validation.html#module-fields"},{"type":"type","title":"Pageantry.Validation.t/0","doc":"","ref":"Pageantry.Validation.html#t:t/0"},{"type":"extras","title":"Pageantry","doc":"# Pageantry\n\nPagination library for Elixir.","ref":"readme.html"},{"type":"extras","title":"Basic Usage - Pageantry","doc":"If you have an Ecto schema like the following:\n\n```\ndefmodule Example.Product do\n  use Ecto.Schema\n\n  schema \"products\" do\n    field :product_name\n    field :quantity_per_unit\n    field :unit_price, :decimal\n    field :units_in_stock, :integer\n    field :units_on_order, :integer\n    field :reorder_level, :integer\n    field :discontinued, :boolean\n    timestamps()\n  end\nend\n```\n\nYou can query a sliced and sorted page of the `products` table with the following `Page.query/4` call:\n\n```\ndefmodule Example.Context do\n  alias Example.Product\n  alias Example.Repo\n  alias Pageantry.{Field, Page}\n\n  def page_products(page \\\\ Page.new()) do\n    Page.query(page, Repo, Product, products_validation())\n  end\n\n  defp products_validation() do\n    fields = [\n      id: %Field{field: :id, all: false},\n      name: %Field{field: :product_name, filter: :like},\n      quantity: %Field{field: :quanity_per_unit, sort: false, filter: false},\n      price: %Field{field: :unit_price},\n      in_stock: %Field{field: :units_in_stock},\n      on_order: %Field{field: :units_on_order},\n      reorder: %Field{field: :reorder_level},\n      created: %Field{field: :inserted_at},\n      updated: %Field{field: :updated_at}\n    ]\n\n    %Validation{schema: Product, fields: fields, base_sort: [asc: :id]}\n  end\nend\n```\n\nWhich you might call from a controller with the pagination input parsed from the request params:\n\n```\ndefmodule ExampleWeb.ProductController do\n  use ExampleWeb, :controller\n  alias Example.Context\n  alias Pageantry.Page\n\n  def index(conn, params) do\n    page =\n      Page.new()\n      |> Page.parse(params)\n      |> Context.page_products()\n\n    render(conn, :index, page: page)\n  end\nend\n```\n\nAnd render with an HTML table like the following:\n\n```\n \n   \n     \n        ID  \n        Product Name  \n       Quantity per Unit \n        Unit Price  \n     \n   \n   \n<%= for item <- @page.output.items do %>\n     \n       <%= item.id %> \n       <%= item.product_name %> \n       <%= item.quantity_per_unit %> \n       <%= item.unit_price %> \n     \n<% end %>\n   \n \n\n \n   Prev \n   Next \n   \n    <%= @page.input.off + 1 %> to <%= @page.input.off + @page.input.max %>\n    of <%= @page.output.total %>\n   \n   \n     \n     Filter \n   \n \n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Join Usage - Pageantry","doc":"If a Product can belong to a Category, like the following:\n\n```\ndefmodule Example.Category do\n  use Ecto.Schema\n  alias Example.Product\n\n  schema \"categories\" do\n    field :category_name\n    field :description\n    field :picture, :binary\n    has_many :products, Product\n  end\nend\n```\n\nYou can add a `query_builder/0` function to the Product schema to define a custom query builder module for it:\n\n```\ndefmodule Example.Product do\n  use Ecto.Schema\n  alias Example.Category\n\n  schema \"products\" do\n    field :product_name\n    field :quantity_per_unit\n    field :unit_price, :decimal\n    field :units_in_stock, :integer\n    field :units_on_order, :integer\n    field :reorder_level, :integer\n    field :discontinued, :boolean\n    belongs_to :category, Category\n    timestamps()\n  end\n\n  def query_builder, do: Example.Product.ProductQueryBuilder\nend\n```\n\nAnd use the query builder to define 1) the alias to use for the `products` table; and 2) the alias and join expression to use to for the `categories` table:\n\n```\ndefmodule Example.Product.ProductQueryBuilder do\n  import Ecto.Query\n  alias Example.{Category, Product}\n\n  def from, do: from(x in Product, as: :products)\n\n  def join(query, Category, qualifier) do\n    join(query, qualifier, [{:products, x}], y in assoc(x, :category), as: :categories)\n  end\nend\n```\n\nThen if you define additional validation fields for a Product using its Category relation:\n\n```\ndefmodule Example.Context do\n  import Ecto.Query\n  alias Example.Product\n  alias Example.Repo\n  alias Pageantry.{Field, FieldRelation, Page}\n\n  def page_products(page \\\\ Page.new()) do\n    query = from(p in Product, as: :products, preload: :category)\n    Page.query(page, Repo, query, products_validation())\n  end\n\n  defp products_validation() do\n    fields = [\n      id: %Field{field: :id, all: false},\n      name: %Field{field: :product_name, filter: :like},\n      quantity: %Field{field: :quanity_per_unit, sort: false, filter: false},\n      price: %Field{field: :unit_price},\n      in_stock: %Field{field: :units_in_stock},\n      on_order: %Field{field: :units_on_order},\n      reorder: %Field{field: :reorder_level},\n      created: %Field{field: :inserted_at},\n      updated: %Field{field: :updated_at},\n      category: %Field{\n        field: :category_name,\n        relation: %FieldRelation{association: :category}\n      },\n      category_description: %Field{\n        field: :description,\n        relation: %FieldRelation{association: :category},\n        filter: :like\n      },\n    ]\n\n    %Validation{schema: Product, fields: fields, base_sort: [asc: :id]}\n  end\nend\n```\n\nYou can sort and filter on the joined Category fields:\n\n```\n \n   \n   \n    Category description:\n     \n   \n   Filter \n \n\n \n   \n     \n        ID  \n        Product Name  \n       Quantity per Unit \n        Unit Price  \n        Category Name  \n       Category Description \n     \n   \n   \n<%= for item <- @page.output.items do %>\n     \n       <%= item.id %> \n       <%= item.product_name %> \n       <%= item.quantity_per_unit %> \n       <%= item.unit_price %> \n       <%= item.category.category_name %> \n       <%= item.category.description %> \n     \n<% end %>\n   \n \n```","ref":"readme.html#join-usage"},{"type":"extras","title":"Development - Pageantry","doc":"1. `make run.db`: start db (for tests)\n2. `make rebuild.elixir`: build dev docker image\n3. `make shell`, then `mix deps.get` (then `exit`): init elixir dev env\n4. `make check`: run linting and tests","ref":"readme.html#development"},{"type":"extras","title":"Resources - Pageantry","doc":"* Source code: https://git.sr.ht/~arx10/elixir-pageantry\n* Documentation: https://hexdocs.pm/pageantry","ref":"readme.html#resources"},{"type":"extras","title":"License - Pageantry","doc":"[The MIT License](https://git.sr.ht/~arx10/elixir-pageantry/tree/main/item/LICENSE)","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}